local M = {}

local vx = 5.0
local vy = 4.0
local threshold = 0.1
local maxX = 1
local minX = -1
local maxY = 1
local minY = -1

local function sign(number)
    return (number > 0 and 1) or (number == 0 and 0) or -1
end

local function onReset()
	electrics.values['shiftknob_x'] = 0
	electrics.values['shiftknob_y'] = 0
	
	electrics.values['currentPos'] = 9
end

local function setShiftingFrom(x, y)
	if x >= maxX-threshold then 
		if y >= maxY-threshold then
			return 1
		elseif (y < maxY-threshold and y >= threshold) then
			return 2
		elseif (y < threshold and y >= -threshold) then
			return 3
		elseif (y < -threshold and y >= -(1-threshold)) then
			return 4
		else
			return 5
		end
	elseif (x < maxX-threshold and x >= threshold) then
		return 6
	elseif (x < threshold and x >= -threshold) then
		if y >= maxY-threshold then
			return 7
		elseif (y < maxY-threshold and y >= threshold) then
			return 8
		elseif (y < threshold and y >= -threshold) then
			return 9
		elseif (y < -threshold and y >= -(1-threshold)) then
			return 10
		else
			return 11
		end
	elseif (x < -threshold and x >= minX + threshold) then
		return 12
	else
		if y >= maxY-threshold then
			return 13
		elseif (y < maxY-threshold and y >= threshold) then
			return 14
		elseif (y < threshold and y >= -threshold) then
			return 15
		elseif (y < -threshold and y >= -(1-threshold)) then
			return 16
		else
			return 17
		end
	end
end

local function setShiftingTo(gear)
	if gear == -1 then
		return 17
	elseif gear == 0 then
		return 9
	elseif gear == 1 then
		return 1
	elseif gear == 2 then
		return 5
	elseif gear == 3 then
		return 7
	elseif gear == 4 then
		return 11
	elseif gear == 5 then
		return 13
	end
end

local function nextStep(currentStep, finalStep)
	if currentStep ~= finalStep then
		if (currentStep == 1 or currentStep == 7 or currentStep == 13) then
			return currentStep + 1
		elseif (currentStep == 5 or currentStep == 11 or currentStep == 17) then
			return currentStep - 1
		elseif currentStep == 2 then
			if finalStep == 1 then
				return 1
			else
				return 3
			end
		elseif currentStep == 3 then
			if finalStep < 3 then
				return 2
			elseif finalStep < 6 then
				return 4
			else
				return 6
			end
		elseif currentStep == 4 then
			if (finalStep < 4 or finalStep > 5) then
				return 3
			else
				return 5
			end
		elseif currentStep == 6 then
			if finalStep < 6 then
				return 3
			else
				return 9
			end
		elseif currentStep == 8 then
			if finalStep == 7 then
				return 7
			else
				return 9
			end
		elseif currentStep == 9 then
			if finalStep <= 6 then
				return 6
			elseif (finalStep > 6 and finalStep < 9) then
				return 8
			elseif (finalStep > 9 and finalStep < 12) then
				return 10
			else
				return 12
			end
		elseif currentStep == 10 then
			if finalStep == 11 then
				return 11
			else
				return 9
			end
		elseif currentStep == 12 then
			if finalStep < 12 then
				return 9
			else
				return 15
			end
		elseif currentStep == 14 then
			if finalStep == 13 then
				return 13
			else
				return 15
			end
		elseif currentStep == 15 then
			if finalStep > 15 then
				return 16
			elseif finalStep > 12 then
				return 14
			else
				return 12
			end
		elseif currentStep == 16 then
			if finalStep < 16 then
				return 15
			else
				return 17
			end
		end
	else
		return finalStep
	end
end

local function getVelocity(currentStep, nextStep)
	if currentStep ~= nextStep then
		if currentStep < 6 then
			if nextStep < 6 then return { 0, vy*(currentStep - nextStep) } else return { -vx, 0 } end
		elseif (currentStep > 6 and currentStep < 12) then
			if (nextStep > 6 and nextStep < 12) then return { 0, vy*(currentStep - nextStep) } else return { vx*(currentStep - nextStep)/3, 0 } end
		elseif currentStep > 12 then
			if nextStep > 12 then return { 0, vy*(currentStep - nextStep) } else return { vx, 0 } end
		else
			return { vx*(currentStep - nextStep)/3, 0 }
		end
	else
		return { 0,0 }
	end
end

local function setShiftPosition(x,y)
	electrics.values['shiftknob_x'] = x
	electrics.values['shiftknob_y'] = y
end

local function updateGFX(dt)
	electrics.values['currentPos'] = setShiftingFrom(electrics.values['shiftknob_x'], electrics.values['shiftknob_y'])
	
	local gear = drivetrain.gear
	local currentPos = electrics.values['currentPos']
	local finalPos = setShiftingTo(gear)
	local nextPos = nextStep(currentPos, finalPos)
	--print("curPos: ")
	--print(currentPos)
	--print("finalPos: ")
	--print(finalPos)
	--print("nextPos: ")
	--print(electrics.values['lights_state'])
	local vel = getVelocity(currentPos, nextPos)
	
	electrics.values['shiftknob_x'] = electrics.values['shiftknob_x']+vel[1]*dt
	electrics.values['shiftknob_y'] = electrics.values['shiftknob_y']+vel[2]*dt
end

M.onInit    = onReset
M.onReset   = onReset
M.updateGFX = updateGFX

return M